require_relative 'playable'

# Players of the game have a name and health.
class Player
  include Playable
  attr_accessor :health
  attr_accessor :name

  ##
  # Create a Player
  #
  # @param name [String] name of the player
  # @param health [Integer] starting health for the player
  def initialize(name, health = 100)
    @name = name.capitalize
    @health = health
    @found_treasures = Hash.new(0)
  end

  ##
  # Override to_s to print a message of player name and health
  def to_s
    "I'm #{name} with health = #{health}, points = #{points}, "\
    "and score = #{score}."
  end

  ##
  # Player's have a score based on health
  #
  # @return [Integer] player's current score
  def score
    @health + points
  end


  ##
  # Override sort operator to sort by score descending
  def <=>(other)
    other.score <=> score
  end

  ##
  # Adds a treasure to the found treasures
  #
  # @param treasure [TreasureTrove::Treasure] treasure to add
  def found_treasure(treasure)
    treasure_name = treasure.name
    treasure_points = treasure.points

    @found_treasures[treasure_name] += treasure_points

    puts "#{@name} found a #{treasure_name} worth #{treasure_points} points."
    puts "#{@name}'s treasures: #{@found_treasures}"
  end

  def points
    @found_treasures.values.reduce(0, :+)
  end

  def each_found_treasure
    @found_treasures.each do |key, value|
      yield TreasureTrove::Treasure.new key, value
    end
  end
end
